from rest_framework import serializers
from django.conf import settings

from openbook_auth.models import UserProfile, User
from openbook_circles.models import Circle
from openbook_common.models import Badge, Language
from openbook_common.serializers import (
    CommonHashtagSerializer,
    CommonPublicUserSerializer,
    LinkPreviewSerializer,
)
from openbook_common.serializers_fields.post import (
    PostCreatorField,
    PostReactionsEmojiCountField,
    ReactionField,
    CommentsCountField,
    CirclesField,
    PostIsMutedField,
    IsEncircledField,
)
from openbook_common.serializers_fields.request import RestrictedFileSizeField, RestrictedImageFileSizeField
from openbook_common.enums import VisibilityType
from openbook_communities.models import CommunityMembership, Community
from openbook_communities.serializers_fields import CommunityMembershipsField
from openbook_posts.models import PostImage, Post
from openbook_posts.validators import post_id_exists, post_text_validators

from openbook_posts.serializers.post_reaction_serializers import PostReactionSerializer
from openbook_posts.serializers.post_reactions_serializers import PostEmojiCountSerializer


class DeletePostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class GetPostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class MutePostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class UnmutePostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class PostCreatorProfileBadgeSerializer(serializers.ModelSerializer):
    class Meta:
        model = Badge
        fields = ("keyword", "keyword_description")


class PostLanguageSerializer(serializers.ModelSerializer):
    class Meta:
        model = Language
        fields = (
            "id",
            "code",
            "name",
        )


class PostCreatorProfileSerializer(serializers.ModelSerializer):
    badges = PostCreatorProfileBadgeSerializer(many=True)

    class Meta:
        model = UserProfile
        fields = ("avatar", "cover", "badges", "name")


class PostImageSerializer(serializers.ModelSerializer):
    image = serializers.ImageField(read_only=True)

    class Meta:
        model = PostImage
        fields = ("image", "width", "height")


class CommunityMembershipSerializer(serializers.ModelSerializer):
    class Meta:
        model = CommunityMembership
        fields = (
            "id",
            "user_id",
            "community_id",
            "is_administrator",
            "is_moderator",
        )


class PostCommunitySerializer(serializers.ModelSerializer):
    memberships = CommunityMembershipsField(community_membership_serializer=CommunityMembershipSerializer)

    class Meta:
        model = Community
        fields = (
            "id",
            "name",
            "title",
            "color",
            "avatar",
            "cover",
            "user_adjective",
            "users_adjective",
            "memberships",
        )


class PostCircleSerializer(serializers.ModelSerializer):
    class Meta:
        model = Circle
        fields = (
            "id",
            "name",
            "color",
        )


class GetPostPostSerializer(serializers.ModelSerializer):
    creator = PostCreatorField(
        post_creator_serializer=CommonPublicUserSerializer,
        community_membership_serializer=CommunityMembershipSerializer,
    )
    reactions_emoji_counts = PostReactionsEmojiCountField(emoji_count_serializer=PostEmojiCountSerializer)
    reaction = ReactionField(reaction_serializer=PostReactionSerializer)
    comments_count = CommentsCountField()
    circles = CirclesField(circle_serializer=PostCircleSerializer)
    community = PostCommunitySerializer()
    is_muted = PostIsMutedField()
    language = PostLanguageSerializer()
    is_encircled = IsEncircledField()
    hashtags = CommonHashtagSerializer(many=True)
    links = LinkPreviewSerializer(many=True)

    class Meta:
        model = Post
        fields = (
            "id",
            "comments_count",
            "reactions_emoji_counts",
            "created",
            "text",
            "creator",
            "reaction",
            "language",
            "comments_enabled",
            "public_reactions",
            "circles",
            "community",
            "hashtags",
            "links",
            "is_muted",
            "is_edited",
            "is_closed",
            "is_encircled",
        )


class EditPostSerializer(serializers.Serializer):
    text = serializers.CharField(
        max_length=settings.POST_MAX_LENGTH,
        required=False,
        allow_blank=True,
        validators=post_text_validators,
    )
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class EditCommunityPostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )
    title = serializers.CharField(
        max_length=settings.POST_TITLE_MAX_LENGTH,
        required=False,
        allow_null=True,
    )
    text = serializers.CharField(
        max_length=settings.POST_MAX_LENGTH,
        required=False,
        allow_blank=False,
        validators=post_text_validators,
    )
    visibility = serializers.ChoiceField(VisibilityType.choices)
    image = RestrictedImageFileSizeField(
        allow_empty_file=False,
        allow_null=True,
        required=False,
        max_upload_size=settings.POST_MEDIA_MAX_SIZE,
    )
    video = RestrictedFileSizeField(
        allow_empty_file=False,
        required=False,
        max_upload_size=settings.POST_MEDIA_MAX_SIZE,
    )
    is_draft = serializers.BooleanField(default=False)


class AuthenticatedUserEditPostSerializer(serializers.ModelSerializer):
    language = PostLanguageSerializer()
    hashtags = CommonHashtagSerializer(many=True)

    class Meta:
        model = Post
        fields = ("id", "text", "language", "is_edited", "hashtags")


class ClosePostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class OpenPostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class TranslatePostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class SearchPostParticipantsSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )
    query = serializers.CharField(max_length=settings.SEARCH_QUERIES_MAX_LENGTH, required=True)
    count = serializers.IntegerField(required=False, default=10, max_value=10)


class GetPostParticipantsSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )
    count = serializers.IntegerField(required=False, default=10, max_value=20)


class PostParticipantProfileSerializer(serializers.ModelSerializer):
    badges = PostCreatorProfileBadgeSerializer(many=True)

    class Meta:
        model = UserProfile
        fields = ("avatar", "cover", "badges", "name")


class PostParticipantSerializer(serializers.ModelSerializer):
    profile = PostCreatorProfileSerializer(many=False)

    class Meta:
        model = User
        fields = ("id", "profile", "username")


class OpenClosePostSerializer(serializers.ModelSerializer):
    class Meta:
        model = Post
        fields = (
            "id",
            "is_closed",
        )


class PublishPostSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )


class GetPostStatusSerializer(serializers.Serializer):
    post_id = serializers.UUIDField(
        validators=[post_id_exists],
        required=True,
    )
